[CCCL] Add missing CCCL components: c2h, nvbench_helper, cmake, cudax, AGENTS.md
Added 863 files from NVIDIA/cccl sparse checkout: - c2h/ (27 files): Catch2 test helpers — generators, validators, runner - nvbench_helper/ (10 files): Benchmark harness utilities - cmake/ (29 files): CMake presets and build helpers - cudax/ (794 files): Experimental CUDA extensions - AGENTS.md: NVIDIA's official AI agent instructions for CCCL - CMakePresets.json: Standardized build configurations - cccl-version.json: Version tracking Also added CCCL_ASSET_MAP.md mapping all 4295 CCCL files to competition value and PRD items. cccl_upstream now covers 100% of competition-critical assets: - 27 tuning headers (SM80/90/100 benchmark data) - 32 dispatch headers (algorithm implementations) - 60 Thrust examples (correctness verification) - 217 CUB Catch2 tests (regression matrix) - 153 CUB benchmarks (parameter space search) - 18 CUB examples (API verification) - 27 test helpers + benchmark harness - 794 cudax experimental extensions
This commit is contained in:
141
cccl_upstream/cudax/test/coop/shuffle/threads_within_warp.cu
Normal file
141
cccl_upstream/cudax/test/coop/shuffle/threads_within_warp.cu
Normal file
@@ -0,0 +1,141 @@
|
||||
//===----------------------------------------------------------------------===//
|
||||
//
|
||||
// Part of CUDA Experimental in CUDA C++ Core Libraries,
|
||||
// under the Apache License v2.0 with LLVM Exceptions.
|
||||
// See https://llvm.org/LICENSE.txt for license information.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2026 NVIDIA CORPORATION & AFFILIATES.
|
||||
//
|
||||
//===----------------------------------------------------------------------===//
|
||||
|
||||
#include <cuda/devices>
|
||||
#include <cuda/hierarchy>
|
||||
#include <cuda/launch>
|
||||
#include <cuda/stream>
|
||||
#include <cuda/type_traits>
|
||||
|
||||
#include <cuda/experimental/coop.cuh>
|
||||
#include <cuda/experimental/group.cuh>
|
||||
|
||||
#include "testing.cuh"
|
||||
|
||||
template <class T>
|
||||
__device__ T make_instance_for(unsigned rank)
|
||||
{
|
||||
if constexpr (cuda::is_vector_type_v<T>)
|
||||
{
|
||||
using U = cuda::scalar_type_t<T>;
|
||||
return T{static_cast<U>(rank), static_cast<U>(rank), static_cast<U>(rank)};
|
||||
}
|
||||
else
|
||||
{
|
||||
return static_cast<T>(rank);
|
||||
}
|
||||
}
|
||||
|
||||
template <class T, class Group>
|
||||
__device__ void test_group(const Group& group)
|
||||
{
|
||||
// Exit all threads that are not part of the group.
|
||||
if (!cuda::gpu_thread.is_part_of(group))
|
||||
{
|
||||
return;
|
||||
}
|
||||
|
||||
const auto my_rank = cuda::gpu_thread.rank_as<unsigned>(group);
|
||||
const auto my_value = make_instance_for<T>(my_rank);
|
||||
|
||||
// Test identity.
|
||||
REQUIRE(cudax::coop::shuffle(group, my_value, my_rank) == my_value);
|
||||
|
||||
// Test broadcast from root rank.
|
||||
REQUIRE(cudax::coop::shuffle(group, my_value, 0) == make_instance_for<T>(0));
|
||||
|
||||
// Test broadcast from last rank.
|
||||
{
|
||||
const auto other_rank = cuda::gpu_thread.count_as<unsigned>(group) - 1;
|
||||
REQUIRE(cudax::coop::shuffle(group, my_value, other_rank) == make_instance_for<T>(other_rank));
|
||||
}
|
||||
|
||||
// Test my_rank + 1.
|
||||
{
|
||||
const auto other_rank = (my_rank + 1) % cuda::gpu_thread.count_as<unsigned>(group);
|
||||
REQUIRE(cudax::coop::shuffle(group, my_value, other_rank) == make_instance_for<T>(other_rank));
|
||||
}
|
||||
|
||||
// Test my_rank + 6.
|
||||
{
|
||||
const auto other_rank = (my_rank + 6) % cuda::gpu_thread.count_as<unsigned>(group);
|
||||
REQUIRE(cudax::coop::shuffle(group, my_value, other_rank) == make_instance_for<T>(other_rank));
|
||||
}
|
||||
|
||||
// Test my_rank +- 1 based on whether my_rank is even or not.
|
||||
{
|
||||
const auto other_rank =
|
||||
((my_rank % 2 == 0) ? my_rank + 1 : my_rank - 1) % cuda::gpu_thread.count_as<unsigned>(group);
|
||||
REQUIRE(cudax::coop::shuffle(group, my_value, other_rank) == make_instance_for<T>(other_rank));
|
||||
}
|
||||
|
||||
// Test my_rank +- 3 based on whether my_rank is even or not.
|
||||
{
|
||||
const auto other_rank =
|
||||
((my_rank % 2 == 0) ? my_rank + 3 : my_rank - 3) % cuda::gpu_thread.count_as<unsigned>(group);
|
||||
REQUIRE(cudax::coop::shuffle(group, my_value, other_rank) == make_instance_for<T>(other_rank));
|
||||
}
|
||||
}
|
||||
|
||||
struct CustomBinaryPartition
|
||||
{
|
||||
template <class MappingResult>
|
||||
__device__ bool operator()(MappingResult mapping_result)
|
||||
{
|
||||
switch (mapping_result.unit_rank())
|
||||
{
|
||||
case 1:
|
||||
case 5:
|
||||
case 14:
|
||||
case 15:
|
||||
case 31:
|
||||
return true;
|
||||
default:
|
||||
return false;
|
||||
}
|
||||
}
|
||||
};
|
||||
|
||||
template <class T, class Config>
|
||||
__device__ void test_type(const Config& config)
|
||||
{
|
||||
const cudax::this_warp warp{config};
|
||||
test_group<T>(cudax::group{cuda::gpu_thread, warp, cudax::identity_mapping{}, cudax::lane_synchronizer{}});
|
||||
test_group<T>(cudax::group{cuda::gpu_thread, warp, cudax::group_by<4>{}, cudax::lane_synchronizer{}});
|
||||
test_group<T>(cudax::group{cuda::gpu_thread, warp, cudax::group_by{1}, cudax::lane_synchronizer{}});
|
||||
test_group<T>(
|
||||
cudax::group{cuda::gpu_thread, warp, cudax::group_by{3, cudax::non_exhaustive}, cudax::lane_synchronizer{}});
|
||||
test_group<T>(
|
||||
cudax::group{cuda::gpu_thread, warp, cudax::binary_partition{CustomBinaryPartition{}}, cudax::lane_synchronizer{}});
|
||||
}
|
||||
|
||||
struct TestKernel
|
||||
{
|
||||
template <class Config>
|
||||
__device__ void operator()(const Config& config)
|
||||
{
|
||||
test_type<signed char>(config);
|
||||
test_type<unsigned>(config);
|
||||
test_type<unsigned long long>(config);
|
||||
test_type<longlong3>(config);
|
||||
}
|
||||
};
|
||||
|
||||
C2H_TEST("shuffle/threads_within_warp", "[shuffle][threads_within_warp]")
|
||||
{
|
||||
const auto device = cuda::devices[0];
|
||||
|
||||
const cuda::stream stream{device};
|
||||
|
||||
const auto config = cuda::make_config(cuda::grid_dims<1>(), cuda::block_dims<32>());
|
||||
cuda::launch(stream, config, TestKernel{});
|
||||
|
||||
stream.sync();
|
||||
}
|
||||
Reference in New Issue
Block a user